'******************************************************************************
'*  tv80.spin - TV text mode with 80x20 characters
'*
'*  This is based on ideas and code from Debug_1pinTV by "Cluso99"
'*
'*  (c) Juergen Buchmueller <pullmoll@t-online.de>
'*
'* $Id: yaz80_demo.spin,v 1.8 2010-03-21 12:43:11 pm Exp $
'******************************************************************************
CON
	_xinfreq		= 5_000_000		' Quartz is 5MHz
	_clkmode		= xtal1 + pll16x	' System clock is 80MHz

#ifndef NTSC
#ifndef PAL
#define PAL
#endif  PAL
#endif  NTSC

#ifdef NTSC
'' NTSC 80x24 using a 6x10 font
ocols			=	80			' number of columns
orows			=	25			' number of rows
ocolsfrq		=	22_478			' Hz per active pixel
oequal1			=	6			' number of equalization pulses
oserr			=	6			' number of serration pulses
oequal2			=	6			' number of equalization pulses
oblank1			=	16			' number of blank lines
oactive			=	25*9			' number of active lines
oblank2			=	10			' number of blank lines
otsync			=	39			' temp sync pulse
ohalf			=	4439			' vscl half line
obackp			=	4190	 		' vscl back porch (sync to active)
ofrontp			=	4158			' vscl front porch (active to sync)
oequal			=	4121			' vscl equalization pulse (sync/2)
osync			=	4146			' vscl sync pulse
osynch			=	4389			' vscl half-sync
oequalh			=	4414			' vscl half-equal
ovsclch			=	4120			' vscl character
ictra			=	12			' CTRA with PLLdiv
ifrqa			=	231701456		' required FRQA value (5MHz*16) (xfrqa replaced by start routine)
ifrqb			=	$4924_0000		' 40/140 << 32 = 1,227,096,064

ifrqa_80MHz		=	289626816		' required FRQA value (5MHz*16)
ifrqa_96MHz		=	241355680		' required FRQA value (6MHz*16)
ifrqa_100MHz		=	231701456		' required FRQA value (6.25MHz*16)
ifrqa_104MHz		=	222789856		' required FRQA value (6.5MHz*16)
ifrqa_108MHz		=	214538384		' required FRQA value (13.5MHz*8)
#endif

'--------------------------------------------------------------------------------------------------
#ifdef PAL
'' PAL 80x24 using a 6x10 font
ocols			=	80			' number of columns
orows			=	25			' number of rows
ocolsfrq		=	22_321			' Hz per active pixel
oequal1			=	4			' number of equalization pulses
oserr			=	5			' number of serration pulses
oequal2			=	5			' number of equalization pulses
oblank1			=	44			' number of blank lines
oactive			=	25*9			' number of active lines
oblank2			=	42			' number of blank lines
otsync			=	39			' temp sync pulse 
ohalf			=	4438			' vscl half line
obackp			=	4184			' vscl back porch (sync to active)
ofrontp			=	4155			' vscl front porch (active to sync)
oequal			=	4120			' vscl equalization pulse (sync/2)
osync			=	4145			' vscl sync pulse
osynch			=	4389			' vscl half-sync
oequalh			=	4414			' vscl half-equal
ovsclch			=	4120			' vscl character
ictra			=	12			' CTRA with PLLdiv
ifrqa			=	287604888		' required FRQA value (5MHz*16) (xfrqa replaced by start routine)
ifrqb			=	$4924_0000		' 40/140 << 32 = 1,227,096,064

ifrqa_80MHz		=	287604888		' required FRQA value (5MHz*16)
ifrqa_96MHz		=	239669904		' required FRQA value (6MHz*16)
ifrqa_100MHz		=	230083104		' required FRQA value (6.25MHz*16)
ifrqa_104MHz		=	221233760		' required FRQA value (6.5MHz*16)
ifrqa_108MHz		=	213039920		' required FRQA value (13.5MHz*8)
#endif
'--------------------------------------------------------------------------------------------------

flash			=	1<<4

deccm			=	%00000001		' DEC cursor mode (0: off, 1: on)
decim			=	%00000010		' DEC insert mode
decom			=	%00000100		' DEC origin mode
deccr			=	%00001000		' DEC send CRLF or LF (0: LF, 1: CRLF)
decck			=	%00010000		' DEC send cursor keys
decawm			=	%00100000		' DEC auto wrap mode
decarm			=	%01000000		' DEC auto repeat mode
decrm			=	%10000000		' DEC report mouse

attr_highlight		=	%00000001
attr_underline		=	%00000010
attr_blinking		=	%00000100
attr_inverse		=	%00001000
attr_fg			=	5			' 3 bits
attr_bg			=	8			' 3 bits

esc_mode_off		=	0
esc_mode_esc		=	1
esc_mode_csi		=	2

VAR
		long	cogs[2]
		long	command
		long	scr_ptr
		long	params[2]
		long	stack[32]
		long	tabs[4]
		long	vtflag				' decXXX bit flags
		long	cur_mode			' cursor mode (24 bits)
		word	attr				' cursor attributes + colors
		word	sav_attr			' saved attributes + colors
		word	cur_pos				' cursor position
		word	cur_row				' cursor row offset
		byte	inverse				' inverse display flag
		byte	dspctl				' display control characters flag
		byte	cur_chr				' character under the cursor
		byte	cur_x				' cursor column
		byte	new_x				' new cursor column
		byte	sav_x				' saved cursor column
		byte	cur_y				' cursor row
		byte	sav_y				' saved cursor row
		byte	scr_top				' scroll range top
		byte	scr_bottom			' scroll range bottom
		byte	data				' data byte sent to the terminal
		byte	esc				' ESC decoding status
		byte	esc_argc			' ESC argument list count
		byte	esc_args[8]			' ESC argument list
		byte	esc_ques			' ESC CSI? flag

PUB start(tvPin, scr) : okay | i
' bitmap is the address of the bitmap buffer * 32 + the tvPin number

	stop
	Setforxtalfreq					'set ifrqa in hub for the current clkfreq before cognew 
	scr_ptr := scr
	params[0] := tvPin
	params[1] := scr
	okay := cogs[0] := COGNEW(@entry, @params) + 1
	okay := cogs[1] := COGNEW(vt100(@command), @stack) + 1	

PUB stop
	if cogs[1]
		COGSTOP(cogs[1] - 1)
		cogs[1] := 0
	if cogs[0]
		COGSTOP(cogs[0] - 1)
		cogs[0] := 0

PUB chr(c)
	command := $100 | c
	repeat while command
				

PUB str(str_ptr) | i
	repeat i from 0 to strsize(str_ptr)
		chr(byte[str_ptr][i])

PUB get_command
	return @command

PRI Setforxtalfreq
'' Set the frqa value depending on the actual clkfreq
	Case clkfreq
		 80_000_000: xfrqa := ifrqa_80MHz	' 5.00 MHz
		 96_000_000: xfrqa := ifrqa_96MHz	' 6.00 MHz
		100_000_000: xfrqa := ifrqa_100MHz	' 6.25 MHz
		104_000_000: xfrqa := ifrqa_104MHz	' 6.50 MHz
		108_000_000: xfrqa := ifrqa_108MHz	'13.50 MHz

PRI vt100(command_ptr) | i
	longfill(@tabs, $01010101, 4)
	clear_range(0, orows * ocols - 1)
	vtflag := deccm
	repeat
		cur_row := cur_y * ocols
		cur_pos := cur_row + new_x
		if new_x < ocols and (vtflag & deccm) == deccm
			cur_chr := byte[scr_ptr][cur_pos]
			byte[scr_ptr][cur_pos] := $7f
		else
			cur_chr := 0
		repeat while long[command_ptr] == 0
		if cur_chr and (vtflag & deccm) == deccm
			byte[scr_ptr][cur_pos] := cur_chr
		data := long[command_ptr] & $7f
		long[command_ptr] := 0
		if esc == esc_mode_esc
			case data
				"[":	' control sequence introducer
					esc := esc_mode_csi
					esc_argc := 0
					bytefill(@esc_args, 0, 8)
				"N":	' <esc>N - select GS0
					vt_si
				"O":	' <esc>O - select GS1
					vt_so
				"D":	' <esc>D - cursor down
					vt_lf
				"E":	' <esc>E - carriage return and cursor down
					vt_crlf
				"M":	' <esc>M - cursor up
					vt_ri
				"H":	' <esc>H - set tabstop
					esc_H
				"Z":	' <esc>Z - respond ID
					' TODO: esc_Z
				"7":	' <esc>7 - save cursor state
					csi_s_lc
				"8":	' <esc>8 - restore cursor state
					csi_u_lc
				"(":	' <esc>( - select GS0
					' TODO: graphics set 0
				")":	' <esc>( - select GS1
					' TODO: graphics set 1
				"#":	' <esc># - DEC alignment test
					' TODO: display pattern?
				"c":	' <esc>c - terminal reset
					esc_c
				">":	' <esc>> - numeric keypad
					' TODO: switch on NumLock
				"=":	' <esc>> - application keypad
					' TODO: switch off NumLock
				other:
					esc := 0
		elseif esc == esc_mode_csi
			esc_ques := 0
			case data
				"?":	' query information
					esc_ques := 1
				";":
					if esc_argc < 7
						esc_argc++
				"0".."9":
					esc_args[esc_argc] *= 10
					esc_args[esc_argc] += data - "0"
				other:
					esc := esc_mode_off
					case data
						"h":	' <esc>[...h - set some state to on
							csi_h_lc
						"l":	' <esc>[...l - set some state to off
							csi_l_lc
						"c":	' <esc>[...c - set cursor mode
							csi_c_lc
						"n":	' <esc>[...n - respond status or cursor
							csi_n_lc
					if esc_ques <> 0
						next
					case data
						"A":		' <esc>[...A - cursor up
							csi_A
						"B","e":	' <esc>[...B - cursor down
								' <esc>[...e - cursor down (variant)
							csi_B
						"C","a":	' <esc>[...C - cursor right
								' <esc>[...a - cursor right (variant)
							csi_C
						"D":		' <esc>[...D - cursor left
							csi_D
						"E":		' <esc>[...D - rows down
							csi_E
						"F":		' <esc>[...D - rows up
							csi_F
						"G","`":	' <esc>[...G - set cursor column
								' <esc>[...` - set cursor column (variant)
							csi_G
						"H":		' <esc>[...H - set cursor position
							csi_H
						"d":		' <esc>[...d - set cursor row
							csi_d_lc
						"J":		' <esc>[...J - clear screen
							csi_J
						"K":		' <esc>[...K - clear cursor row
							csi_K
						"L":		' <esc>[...L - insert line(s)
							csi_L
						"M":		' <esc>[...M - delete line(s)
							csi_M
						"P":		' <esc>[...P - delete character(s)
							csi_P
						"g":		' <esc>[...g - reset tabstop
							csi_g_lc
						"m":		' <esc>[...m - set or reset attribute(s)
							' csi_m_lc
						"q":		' <esc>[...q - DEC set LEDs
							' TODO: csi_q_lc
						"r":		' <esc>[...r - set scroll region
							csi_r_lc
						"s":		' <esc>[...s - save cursor
							csi_s_lc
						"u":		' <esc>[...u - restore cursor
							csi_u_lc
						"X":		' <esc>[...X - blank characters
							csi_X
						"@":		' <esc>[...@ - insert blanks
							csi_AT
		else
			case data
				8:	' back space
					if cur_x > 0
						cur_x--
					else
						cur_x := ocols - 1
						vt_ri
				9:	' tab
					vt_tab
				10:	' line feed
					vt_lf
				11:	' vertical tab
					vt_ri
				12:	' form feed
					vt_ff
				13:	' carriage return
					vt_cr
				14:	' shift out
					vt_so
				15:	' shift in
					vt_si
				27:	' escape
					esc := esc_mode_esc
				other:
					if data => 32 and data < 127
						emit(data)

PRI emit(ch)
	cur_x := new_x
	byte[scr_ptr][cur_row + cur_x] := ch
	new_x := cur_x + 1

'*****************************************************************
' gotoxy make sure the cursor coordinates are within the bounds
'
PRI gotoxy
	cur_x := (cur_x #> 0) <# (ocols - 1)
	if vtflag & decom
		cur_y := (cur_y #> scr_top) <# (scr_bottom - 1)
	else
		cur_y := (cur_y #> 0) <# (orows - 1)

'*****************************************************************
' gotoxay clamp cursor coordinates relative to scr_top if decom
' is set in the vtflag
'
PRI gotoxay
	if vtflag & decom
		cur_y += scr_top
	gotoxy

'*****************************************************************
' clear_range fill a range of the screen with spaces (blanks)
'
PRI clear_range(beg, end)
	bytefill(scr_ptr + beg, $20, end + 1 - beg)

'*****************************************************************
' scroll_dn scroll the entire screen down by a line
'
PRI scroll_dn(top, bottom)
	if bottom - top - 1 > 0
		longmove(scr_ptr + top * ocols + ocols, scr_ptr + top * ocols, (bottom - top - 1) * ocols / 4)
	longfill(scr_ptr + top * ocols, $20202020, ocols / 4)

'*****************************************************************
' scroll_up scroll the entire screen up by a line
'
PRI scroll_up(top, bottom)
	if bottom - top - 1 > 0
		longmove(scr_ptr + top * ocols, scr_ptr + top * ocols + ocols, (bottom - top - 1) * ocols / 4)
	longfill(scr_ptr + (bottom - 1) * ocols, $20202020, ocols / 4)

'*****************************************************************
' vt_lf line feed
'
PRI vt_lf
	cur_y++
	if vtflag & decom
		if cur_y == scr_bottom
			scroll_up(scr_top, scr_bottom)
			cur_y := scr_bottom - 1
	else
		if cur_y == orows
			scroll_up(0, orows)
			cur_y := orows - 1

'*****************************************************************
' vt_cr carriage return
'
PRI vt_cr
	new_x := cur_x := 0

'*****************************************************************
' vt_crlf carriage return and line feed
'
PRI vt_crlf
	vt_cr
	vt_lf

'*****************************************************************
' vt_ri reverse line feed
'
PRI vt_ri
	if vtflag & decom
		if cur_y > scr_top
			cur_y--
		else
			scroll_dn(scr_top, scr_bottom)
	else
		if cur_y > 0
			cur_y--
		else
			scroll_dn(0, orows)

'*****************************************************************
' vt_ff form feed - clear screen and cursor hom
'
PRI vt_ff
	clear_range(0, orows*ocols-1)
	cur_x := 0
	new_x := 0
	cur_y := 0

'*****************************************************************
' vt_tab emit blanks until the next tab stop
'
PRI vt_tab
	repeat until ((tabs[cur_x >> 5] >> (cur_x & 31)) & 1) == 1 or cur_x == ocols - 1
		emit(" ")

'*****************************************************************
' vt_si shift in - select graphics set 0 (GS0)
'
PRI vt_si
	' TODO: graphics set support
	' Would require 4 translation tables
	' 0: Latin1 to unicode
	' 1: DEC to unicode
	' 2: CP437 to unicode
	' 3: User to unicode

'*****************************************************************
' vt_so shift out - select graphics set 1 (GS1)
'
PRI vt_so
	' TODO: graphics set support

'*****************************************************************
' esc_H set tab stop at cursor column
'
PRI esc_H
	tabs[cur_x >> 5] |= (1 << (cur_x & 31))

'*****************************************************************
' esc_c reset terminal
'
PRI esc_c
	clear_range(0, orows * ocols - 1)
	cur_x := cur_y := 0
	vtflag := 0
	inverse := 0
	longfill(@tabs, $01010101, 4)

'*****************************************************************
' csi_A cursor up
'
PRI csi_A
	cur_y -= esc_args[0] #> 1
	gotoxy

'*****************************************************************
' csi_B cursor down
'
PRI csi_B
	cur_y += esc_args[0] #> 1
	gotoxy

'*****************************************************************
' csi_C cursor right
'
PRI csi_C
	cur_x := new_x + esc_args[0] #> 1
	gotoxy

'*****************************************************************
' csi_D cursor left
'
PRI csi_D
	cur_x := new_x - esc_args[0] #> 1
	gotoxy

'*****************************************************************
' csi_E rows up
'
PRI csi_E
	cur_x := 0
	cur_y -= esc_args[0] #> 1
	gotoxy

'*****************************************************************
' csi_F rows down
'
PRI csi_F
	cur_x := new_x
	cur_y += esc_args[0] #> 1
	gotoxy

'*****************************************************************
' csi_G set cursor column
'
PRI csi_G
	cur_x := (esc_args[0] #> 1) - 1
	gotoxy

'*****************************************************************
' csi_H set cursor address
'
PRI csi_H
	cur_y := (esc_args[0] #> 1) - 1
	cur_x := (esc_args[1] #> 1) - 1
	gotoxy

'*****************************************************************
' csi_J clear a range of the screen
'
PRI csi_J
	case esc_args[0]
		0:	' clear from cursor to end of screen
			clear_range(cur_pos, orows*ocols-1)
		1:	' clear from start of screen to cursor
			clear_range(0, cur_pos)
		2:	' clear entire screen
			clear_range(0, orows*ocols-1)

'*****************************************************************
' csi_L clear a range of the cursor row
'
PRI csi_K
	case esc_args[0]
		0:	' clear from cursor to end of row
			clear_range(cur_pos, cur_row + ocols - 1)
		1:	' clear from start of row to cursor
			clear_range(cur_row, cur_pos)
		2:	' clear entire row
			clear_range(cur_row, cur_row + ocols - 1)

'*****************************************************************
' csi_L insert one or more empty rows at the cursor row
'
PRI csi_L
	esc_args[0] := esc_args[0] #> 1
	repeat while esc_args[0]--
		if cur_y < orows - 1
			longmove(scr_ptr + cur_row + ocols, scr_ptr + cur_row, (orows - cur_y - 1) * ocols / 4)
		clear_range(cur_row, cur_row + ocols - 1)

'*****************************************************************
' csi_M delete one or more rows at the cursor row
'
PRI csi_M
	esc_args[0] := esc_args[0] #> 1
	repeat while esc_args[0]--
		if cur_y < orows - 1
			longmove(scr_ptr + cur_row, scr_ptr + cur_row + ocols, (orows - cur_y - 1) * ocols / 4)
		clear_range((orows - 1) * ocols, orows * ocols - 1)

'*****************************************************************
' csi_P delete one or more characters at the cursor position
'
PRI csi_P
	esc_args[0] := esc_args[0] #> 1
	repeat while esc_args[0]--
		if cur_x < ocols - 1
			bytemove(scr_ptr + cur_pos, scr_ptr + cur_pos + 1, ocols - cur_x - 1)
		byte[scr_ptr][cur_row + ocols - 1] := $20

'*****************************************************************
' csi_X blank a number of characters
'
PRI csi_X
	clear_range(cur_pos, (cur_pos + esc_args[0]) <# ocols - 1)

'*****************************************************************
' csi_AT insert blank characters
'
PRI csi_AT
	esc_args[0] := esc_args[0] #> 1
	repeat while esc_args[0]--
		if cur_x < ocols - 1
			bytemove(scr_ptr + cur_pos + 1, scr_ptr + cur_pos, ocols - cur_x - 1)
		byte[scr_ptr][cur_row + cur_pos] := $20

'*****************************************************************
' csi_h set some state to on
'
PRI csi_h_lc | i
	repeat i from 0 to esc_argc-1
		if esc_ques
			case esc_args[i]
				1:	' <esc>[?1h set cursor keys mode on
					vtflag |= decck
				2:	' <esc>[?2h set 132 column mode
					' Bwahahaha!
				5:	' <esc>[?5h set screen inverse mode on
					inverse := 1
				6:	' <esc>[?6h DEC origin mode on
					vtflag |= decom
					cur_x := 0
					cur_y := 0
					gotoxay
				7:	' <esc>[?7h DEC auto wrap mode on
					vtflag |= decawm
				8:	' <esc>[?8h DEC auto repeat mode on
					vtflag |= decarm
				9:	' <esc>[?9h DEC report mouse mode on
					vtflag |= decrm
				25:	' <esc>[?25h DEC cursor mode on
					vtflag |= deccm
		else
			case esc_args[i]
				3:	' <esc>[3h control display mode on
					dspctl := 1
				4:	' <esc>[4h DEC insert mode on
					vtflag |= decim
				5:	' <esc>[5h DEC auto CR mode on
					vtflag |= deccr

'*****************************************************************
' csi_l set some state to off
'
PRI csi_l_lc | i
	repeat i from 0 to esc_argc-1
		if esc_ques
			case esc_args[i]
				1:	' <esc>[?1h set cursor keys mode on
					vtflag &= !decck
				2:	' <esc>[?2h set 80 column mode
					' anyway
				5:	' <esc>[?5h set screen inverse mode off
					inverse := 0
				6:	' <esc>[?6h DEC origin mode off
					vtflag &= !decom
					cur_x := 0
					cur_y := 0
					gotoxay
				7:	' <esc>[?7h DEC auto wrap mode off
					vtflag &= !decawm
				8:	' <esc>[?8h DEC auto repeat mode off
					vtflag &= !decarm
				9:	' <esc>[?9h DEC report mouse mode off
					vtflag &= !decrm
				25:	' <esc>[?25h DEC cursor mode off
					vtflag &= !deccm
		else
			case esc_args[i]
				3:	' <esc>[3h control display mode off
					dspctl := 0
				4:	' <esc>[4h DEC insert mode off
					vtflag &= !decim
				5:	' <esc>[5h DEC auto CR mode off
					vtflag &= !deccr

'*****************************************************************
' csi_c set cursor mode
'
PRI csi_c_lc | i
	if esc_ques == 0
		cur_mode := esc_args[0] + esc_args[1] << 8 + esc_args[2] << 16

'*****************************************************************
' csi_d set cursor row (relative to scroll range if decom)
'
PRI csi_d_lc
	cur_y := (esc_args[0] #> 1) - 1
	gotoxay

'*****************************************************************
' csi_g reset tabstop
'
PRI csi_g_lc
	if new_x => 0 and new_x < ocols
		tabs[new_x >> 5] &= !(1 << (new_x & 31))
				
'*****************************************************************
' csi_m set one or more mode attributes
'
PRI csi_m_lc | i
	repeat i from 0 to esc_argc-1
		case esc_args[i]
			0:	' reset all
				attr := 0
			1:	' highlight on
				attr |= attr_highlight
			2:	' highlight off
				attr &= !attr_highlight
			4:	' underline on
				attr |= attr_underline
			5:	' blinking on
				attr |= attr_blinking
			7:	' inverse on
				attr |= attr_inverse
			10:	' primary font, no ctrl, no meta
				' TODO
			11:	' alternate font, ctrl chars, low half, meta off
				' TODO
			12:	' alternate font, ctrl chars, high half, mea on
				' TODO
			21..22:	' highlight on
				attr |= attr_highlight
			24:	' underline off
				attr &= !attr_underline
			25:	' blinking off
				attr &= !attr_blinking
			27:	' inverse off
				attr &= !attr_inverse
			30..37:	' foreground color 0..7
				attr := (attr & !(7 << attr_fg)) | (esc_args[i] - 30) << attr_fg
			38:	' default color and underline on
				attr |= 7 << attr_fg
				attr |= attr_underline
			39:	' default color and underline off
				attr |= 7 << attr_fg
				attr &= !attr_underline
			40:	' default background
				attr := (attr & !(7 << attr_bg))
			41..47:	' background color 1..7
				attr := (attr & !(7 << attr_bg)) | (esc_args[i] - 40) << attr_bg
			49:	' default background
				attr := (attr & !(7 << attr_bg))

'*****************************************************************
' csi_n respond status or cursor
'
PRI csi_n_lc | i
	repeat i from 0 to esc_argc-1
		case esc_args[i]
			5:	' <esc>[5n - respond status
				' TODO
			6:	' <esc>[6n - respond cursor

'*****************************************************************
' csi_r set scrolling region
'
PRI csi_r_lc
	if esc_argc < 2
		scr_top := 0
		scr_bottom := orows
	else
		scr_top := (esc_args[0] #> 1) - 1
		scr_bottom := (esc_args[1] #> 1) - 1
		if scr_top > scr_bottom or scr_bottom > orows
			' invalid range - reset it
			scr_top := 0
			scr_bottom := orows - 1
	cur_x := 0
	cur_y := 0
	gotoxay

'*****************************************************************
' csi_s_lc save cursor
'
PRI csi_s_lc
	sav_x := cur_x
	sav_y := cur_y
	sav_attr := attr

'*****************************************************************
' csi_u_lc restore cursor
'
PRI csi_u_lc
	cur_x := sav_x
	cur_y := sav_y
	attr := sav_attr

DAT
		org	0
screen		byte
entry
		mov	t1, PAR
		rdlong	pin_no, t1
		add	t1, #4
		rdlong	screen_ptr, t1
		add	screen_end, screen_ptr

		mov	FRQB, xfrqb			' default = 40/140

		movi	CTRA, #ictra
		mov	FRQA, xfrqa

		movs	CTRB, pin_no			' set pin
		mov	count, #1
		shl	count, pin_no
		mov	DIRA, count			' set pin mask
		mov	count, pin_no
		shr	count, #3
		movd	VCFG, count			' set VGroup
		mov	count, #1
		and	pin_no, #7
		shl	count, pin_no
		movs	VCFG, count			' set VPins
		movi	VCFG, #%0_01_0_0_0_000		' VGA mode, 1 bit per pixel

'**************************************************************************************************
frame		add	framectr, #1			' inc frame counter
		mov	rows, #oequal1
		call	#equalizing
'--------------------------------------------------------------------------------------------------
		mov	rows, #oserr			' serration pulses
:serration
		mov	vscl, xsynch			' serration pulse (long)
		waitvid xff00, #0		
		movi	CTRB, #0			' turn off blank
		mov	vscl, xsync			' serration pulse (short)
		waitvid xff00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		djnz	rows, #:serration
'--------------------------------------------------------------------------------------------------
		mov	rows, #oequal2
		call	#equalizing
'==================================================================================================
		mov	rows, #oblank1
		call	#doblank
'==================================================================================================
'display active lines
		mov	fontindex, font_ptr
		mov	scrptr0, screen_ptr		' initialize bitmap pointer
		mov	rows, #oactive
doactive
		mov	vscl, xsync			' 4.7uSec @ -40 IRE
		waitvid xff00, #0
		movi	CTRB, #0			' turn off blank
		mov	count, #ocols/4			' characters per line / 4, since we do 4 at a time
		mov	vscl, xbackp			' back porch
		waitvid xff00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		mov	vscl, xvsclch			' 1 PLLA per pixel, 6 PLLA per frame
		mov	scrptr, scrptr0
'--------------------------------------------------------------------------------------------------
:active
		rdlong	ch2, scrptr			' get 4 characters
		add	scrptr, #4

		mov	ch1, ch2
		and	ch1, #%01111111
		add	ch1, fontindex			' + fontindex = address of font row
		rdbyte	bits0, ch1			' read to bits0

		ror	ch2, #8
		mov	ch1, ch2
		and	ch1, #%01111111
		add	ch1, fontindex			' + fontindex = address of font row
		rdbyte	bits1, ch1			' read to bits1
		shl	bits1, #1*6			' to bits 11..6

		ror	ch2, #8
		mov	ch1, ch2
		and	ch1, #%01111111
		add	ch1, fontindex			' + fontindex = address of font row
		rdbyte	bits2, ch1			' read to bits2
		shl	bits2, #2*6			' to bits 17..12

		ror	ch2, #8
		and	ch2, #%01111111
		add	ch2, fontindex			' + fontindex = address of font row
		rdbyte	bits3, ch2			' read to bits3
		shl	bits3, #3*6			' to bits 23..18

		or	bits0, bits1
		or	bits0, bits2
		or	bits0, bits3

		waitvid	x00ff, bits0			' output 24 bits to screen
		djnz	count, #:active			' do entire line
'--------------------------------------------------------------------------------------------------

		add	fontindex, #128
		cmp	fontindex, font_end	WZ
	if_z	mov	fontindex, font_ptr
	if_z	add	scrptr0, #ocols

		mov	vscl, xfrontp			' front porch
		waitvid xff00, #0			' output blank
		djnz	rows, #doactive			' next row
'==================================================================================================
		mov	rows, #oblank2
		call	#doblank
'==================================================================================================
		jmp	#frame

doblank
		mov	vscl, xsync			' 4.7uSec @ -40 IRE
		waitvid xff00, #0
		movi	CTRB, #0			' turn off blank
		mov	vscl, xsynch			' blank line
		waitvid xff00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		mov	vscl, xhalf			' half line ???
		waitvid xff00, #0
		djnz	rows, #doblank
doblank_ret
		ret

equalizing
		mov	vscl, xequal			' equalizing pulse (short)
		waitvid xff00, #0
		movi	CTRB, #0			' turn off blank
		mov	vscl, xequalh			' equalizing pulse (long)
		waitvid xff00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		djnz	rows, #equalizing
equalizing_ret
		ret
'--------------------------------------------------------------------------------------------------
'constants > $1FF
xhalf		long	ohalf
xbackp		long	obackp
xfrontp		long	ofrontp
xequal		long	oequal
xsync		long	osync
xsynch		long	osynch
xequalh		long	oequalh
xvsclch		long	ovsclch
xfrqa		long	ifrqa
xfrqb		long	ifrqb

xff00		long	$ff00			' white / black
x00ff		long	$00ff			' white / black
low_word	long	$ffff
x20202020	long	$20202020
x7f		long	$7f
font_ptr	long	@@@font
font_end	long	@@@font + 128*9
pin_no		long	0
screen_ptr	long	0-0
screen_end	long	ocols*orows*2
framectr	long	0

t1		res	1
ch1		res	1
ch2		res	1
bits0		res	1			' 1st bit mask
bits1		res	1			' 2nd bit mask
bits2		res	1			' 3rd bit mask
bits3		res	1			' 4th bit mask
fontindex	res	1
scrptr0		res	1			' pointer to screen data per character row
scrptr		res	1			' pointer to screen data per scanline
count		res	1			' all purpose counter
rows		res	1			' row counter
'**************************************************************************************************
		fit	$1f0

'**************************************************************************************************
font
				' scanrow 0
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%000100	' ..#...
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%100000	' .....#
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%010100	' ..#.#.
		byte	%010100	' ..#.#.
		byte	%001000	' ...#..
		byte	%000110	' .##...
		byte	%000100	' ..#...
		byte	%001100	' ..##..
		byte	%010000	' ....#.
		byte	%000100	' ..#...
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%010000	' ....#.
		byte	%111110	' .#####
		byte	%011000	' ...##.
		byte	%111110	' .#####
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%011110	' .####.
		byte	%011100	' ..###.
		byte	%011110	' .####.
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%111100	' ..####
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%011110	' .####.
		byte	%011100	' ..###.
		byte	%011110	' .####.
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%111110	' .#####
		byte	%111100	' ..####
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%000000	' ......
		byte	%011000	' ...##.
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%100000	' .....#
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%001000	' ...#..
		byte	%000100	' ..#...
		byte	%000100	' ..#...
		byte	%010100	' ..#.#.

				' scanrow 1
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%000100	' ..#...
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%101000	' ...#.#
		byte	%010100	' ..#.#.
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%010100	' ..#.#.
		byte	%010100	' ..#.#.
		byte	%111100	' ..####
		byte	%100110	' .##..#
		byte	%001010	' .#.#..
		byte	%001100	' ..##..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%001100	' ..##..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%011000	' ...##.
		byte	%000010	' .#....
		byte	%000100	' ..#...
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%010100	' ..#.#.
		byte	%100100	' ..#..#
		byte	%100010	' .#...#
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%010010	' .#..#.
		byte	%000010	' .#....
		byte	%110110	' .##.##
		byte	%100110	' .##..#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100000	' .....#
		byte	%001100	' ..##..
		byte	%000010	' .#....
		byte	%001111	' ####..
		byte	%001010	' .#.#..
		byte	%000000	' ......
		byte	%011000	' ...##.
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%100000	' .....#
		byte	%000000	' ......
		byte	%101000	' ...#.#
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#

				' scanrow 2
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%010000	' ....#.
		byte	%110110	' .##.##
		byte	%010000	' ....#.
		byte	%100010	' .#...#
		byte	%011110	' .####.
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%000100	' ..#...
		byte	%110110	' .##.##
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%010000	' ....#.
		byte	%010100	' ..#.#.
		byte	%100000	' .....#
		byte	%010100	' ..#.#.
		byte	%011100	' ..###.
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%010100	' ..#.#.
		byte	%111110	' .#####
		byte	%001010	' .#.#..
		byte	%010000	' ....#.
		byte	%001010	' .#.#..
		byte	%000100	' ..#...
		byte	%000100	' ..#...
		byte	%010000	' ....#.
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%110010	' .#..##
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%100000	' .....#
		byte	%010100	' ..#.#.
		byte	%011110	' .####.
		byte	%000010	' .#....
		byte	%010000	' ....#.
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001100	' ..##..
		byte	%001100	' ..##..
		byte	%000100	' ..#...
		byte	%111110	' .#####
		byte	%010000	' ....#.
		byte	%100000	' .....#
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%001010	' .#.#..
		byte	%000010	' .#....
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%010100	' ..#.#.
		byte	%010100	' ..#.#.
		byte	%010000	' ....#.
		byte	%001100	' ..##..
		byte	%000100	' ..#...
		byte	%001100	' ..##..
		byte	%010001	' #...#.
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%011010	' .#.##.
		byte	%011100	' ..###.
		byte	%101100	' ..##.#
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%101100	' ..##.#
		byte	%011010	' .#.##.
		byte	%001100	' ..##..
		byte	%100000	' .....#
		byte	%010010	' .#..#.
		byte	%001000	' ...#..
		byte	%010110	' .##.#.
		byte	%011010	' .#.##.
		byte	%011100	' ..###.
		byte	%011010	' .#.##.
		byte	%101100	' ..##.#
		byte	%011010	' .#.##.
		byte	%111100	' ..####
		byte	%111110	' .#####
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%010000	' ....#.
		byte	%010100	' ..#.#.

				' scanrow 3
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%111100	' ..####
		byte	%101010	' .#.#.#
		byte	%001010	' .#.#..
		byte	%111110	' .#####
		byte	%100100	' ..#..#
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%111110	' .#####
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%111110	' .#####
		byte	%111010	' .#.###
		byte	%111010	' .#.###
		byte	%101110	' .###.#
		byte	%101110	' .###.#
		byte	%101010	' .#.#.#
		byte	%010100	' ..#.#.
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%000100	' ..#...
		byte	%111110	' .#####
		byte	%101110	' .###.#
		byte	%101110	' .###.#
		byte	%111010	' .#.###
		byte	%111010	' .#.###
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%010100	' ..#.#.
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%000100	' ..#...
		byte	%000010	' .#....
		byte	%000100	' ..#...
		byte	%010000	' ....#.
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%011000	' ...##.
		byte	%010010	' .#..#.
		byte	%100000	' .....#
		byte	%011110	' .####.
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%111100	' ..####
		byte	%000000	' ......
		byte	%001100	' ..##..
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%100000	' .....#
		byte	%010000	' ....#.
		byte	%101100	' ..##.#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%000010	' .#....
		byte	%100100	' ..#..#
		byte	%001110	' .###..
		byte	%001110	' .###..
		byte	%110010	' .#..##
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%000110	' .##...
		byte	%000010	' .#....
		byte	%101010	' .#.#.#
		byte	%110010	' .#..##
		byte	%100010	' .#...#
		byte	%011110	' .####.
		byte	%100010	' .#...#
		byte	%011110	' .####.
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%010100	' ..#.#.
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001100	' ..##..
		byte	%001000	' ...#..
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%100000	' .....#
		byte	%100110	' .##..#
		byte	%100010	' .#...#
		byte	%110010	' .#..##
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%110010	' .#..##
		byte	%100110	' .##..#
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001010	' .#.#..
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%100110	' .##..#
		byte	%100010	' .#...#
		byte	%100110	' .##..#
		byte	%110010	' .#..##
		byte	%100110	' .##..#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%010100	' ..#.#.
		byte	%100010	' .#...#
		byte	%010000	' ....#.
		byte	%000100	' ..#...
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%101010	' .#.#.#

				' scanrow 4
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%001000	' ...#..
		byte	%110110	' .##.##
		byte	%000110	' .##...
		byte	%010100	' ..#.#.
		byte	%101000	' ...#.#
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%101010	' .#.#.#
		byte	%000100	' ..#...
		byte	%110110	' .##.##
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%000110	' .##...
		byte	%010100	' ..#.#.
		byte	%100000	' .....#
		byte	%010100	' ..#.#.
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%111110	' .#####
		byte	%101000	' ...#.#
		byte	%000100	' ..#...
		byte	%101010	' .#.#.#
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%010000	' ....#.
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%100110	' .##..#
		byte	%001000	' ...#..
		byte	%000010	' .#....
		byte	%100000	' .....#
		byte	%111110	' .#####
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%000100	' ..#...
		byte	%100010	' .#...#
		byte	%100000	' .....#
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%111110	' .#####
		byte	%010000	' ....#.
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%111110	' .#####
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%001010	' .#.#..
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%101010	' .#.#.#
		byte	%001010	' .#.#..
		byte	%100000	' .....#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%010100	' ..#.#.
		byte	%101010	' .#.#.#
		byte	%010100	' ..#.#.
		byte	%001000	' ...#..
		byte	%000100	' ..#...
		byte	%001100	' ..##..
		byte	%010000	' ....#.
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%111100	' ..####
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%110010	' .#..##
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%000110	' .##...
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%010100	' ..#.#.

				' scanrow 5
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%010000	' ....#.
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%010100	' ..#.#.
		byte	%100000	' .....#
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%010100	' ..#.#.
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%010100	' ..#.#.
		byte	%011110	' .####.
		byte	%110010	' .#..##
		byte	%010010	' .#..#.
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%001100	' ..##..
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%010000	' ....#.
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%010000	' ....#.
		byte	%001100	' ..##..
		byte	%001100	' ..##..
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100100	' ..#..#
		byte	%100010	' .#...#
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%010010	' .#..#.
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%010010	' .#..#.
		byte	%010010	' .#..#.
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%110110	' .##.##
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%000010	' .#....
		byte	%001100	' ..##..
		byte	%100000	' .....#
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%100010	' .#...#
		byte	%100110	' .##..#
		byte	%100010	' .#...#
		byte	%110010	' .#..##
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%101100	' ..##.#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%001010	' .#.#..
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100110	' .##..#
		byte	%110010	' .#..##
		byte	%000010	' .#....
		byte	%100000	' .....#
		byte	%101000	' ...#.#
		byte	%110010	' .#..##
		byte	%010100	' ..#.#.
		byte	%101010	' .#.#.#
		byte	%010100	' ..#.#.
		byte	%111100	' ..####
		byte	%000100	' ..#...
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%101010	' .#.#.#

				' scanrow 6
		byte	%111110	' .#####
		byte	%000010	' .#....
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%100000	' .....#
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%110110	' .##.##
		byte	%100000	' .....#
		byte	%000000	' ......
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%110110	' .##.##
		byte	%100000	' .....#
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%010100	' ..#.#.
		byte	%001000	' ...#..
		byte	%110000	' ....##
		byte	%101100	' ..##.#
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%000100	' ..#...
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%011100	' ..###.
		byte	%010000	' ....#.
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%000010	' .#....
		byte	%011100	' ..###.
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%001100	' ..##..
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%100010	' .#...#
		byte	%011110	' .####.
		byte	%011100	' ..###.
		byte	%011110	' .####.
		byte	%111110	' .#####
		byte	%000010	' .#....
		byte	%111100	' ..####
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%100010	' .#...#
		byte	%111110	' .#####
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%000010	' .#....
		byte	%101100	' ..##.#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%111110	' .#####
		byte	%111100	' ..####
		byte	%000000	' ......
		byte	%001111	' ####..
		byte	%000000	' ......
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%111100	' ..####
		byte	%011010	' .#.##.
		byte	%011100	' ..###.
		byte	%101100	' ..##.#
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%100000	' .....#
		byte	%010010	' .#..#.
		byte	%011100	' ..###.
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%011010	' .#.##.
		byte	%101100	' ..##.#
		byte	%000010	' .#....
		byte	%011110	' .####.
		byte	%010000	' ....#.
		byte	%101100	' ..##.#
		byte	%001000	' ...#..
		byte	%010100	' ..#.#.
		byte	%100010	' .#...#
		byte	%100000	' .....#
		byte	%111110	' .#####
		byte	%010000	' ....#.
		byte	%001000	' ...#..
		byte	%000100	' ..#...
		byte	%000000	' ......
		byte	%010100	' ..#.#.

				' scanrow 7
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%100010	' .#...#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%100010	' .#...#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%100000	' .....#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%100010	' .#...#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......

				' scanrow 8
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%100000	' .....#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......


'**************************************************************************************************

{{
+------------------------------------------------------------------------------------------------------------------------------+
|				    TERMS OF USE: Parallax Object Exchange License					       |
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | |files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE	       |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}
